@@ -2,12 +2,15 @@ package github.paroj.dsub2000.fragments;
 
 import android.annotation.TargetApi;
 import androidx.appcompat.app.AlertDialog;
+
 import android.content.DialogInterface;
 import android.content.Intent;
 import android.content.SharedPreferences;
 import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
+
+import androidx.lifecycle.ViewModelProvider;
 import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;
 import androidx.recyclerview.widget.GridLayoutManager;
 import androidx.recyclerview.widget.LinearLayoutManager;
@@ -44,7 +47,6 @@ import github.paroj.dsub2000.service.DownloadService;
 import github.paroj.dsub2000.util.DrawableTint;
 import github.paroj.dsub2000.util.ImageLoader;
 
-import java.io.Serializable;
 import java.util.List;
 
 import github.paroj.dsub2000.domain.PodcastEpisode;
@@ -52,6 +54,7 @@ import github.paroj.dsub2000.service.MusicService;
 import github.paroj.dsub2000.service.MusicServiceFactory;
 import github.paroj.dsub2000.service.OfflineException;
 import github.paroj.dsub2000.service.ServerTooOldException;
+import github.paroj.dsub2000.domain.repository.StarredRepository;
 import github.paroj.dsub2000.util.Constants;
 import github.paroj.dsub2000.util.LoadingTask;
 import github.paroj.dsub2000.util.Pair;
@@ -64,6 +67,7 @@ import github.paroj.dsub2000.view.FastScroller;
 import github.paroj.dsub2000.view.MyLeadingMarginSpan2;
 import github.paroj.dsub2000.view.RecyclingImageView;
 import github.paroj.dsub2000.view.UpdateView;
+import github.paroj.dsub2000.viewmodel.SelectDirectoryViewModel;
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -76,6 +80,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 	private static final String TAG = SelectDirectoryFragment.class.getSimpleName();
 
 	private RecyclerView recyclerView;
+    private SelectDirectoryViewModel viewModel;
 	private FastScroller fastScroller;
 	private EntryGridAdapter entryGridAdapter;
 	private Boolean licenseValid;
@@ -117,13 +122,15 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 	@Override
 	public void onCreate(Bundle bundle) {
 		super.onCreate(bundle);
+
+	    viewModel = new ViewModelProvider(this).get(SelectDirectoryViewModel.class);
 		if(bundle != null) {
-			entries = (List<Entry>) bundle.getSerializable(Constants.FRAGMENT_LIST);
-			albums = (List<Entry>) bundle.getSerializable(Constants.FRAGMENT_LIST2);
+            entries = viewModel.getEntries().getValue();
+            albums = viewModel.getAlbums().getValue();
 			if(albums == null) {
 				albums = new ArrayList<>();
 			}
-			artistInfo = (ArtistInfo) bundle.getSerializable(Constants.FRAGMENT_EXTRA);
+            artistInfo = viewModel.getArtistInfo().getValue();
 			restoredInstance = true;
 		}
 	}
@@ -131,9 +138,9 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 	@Override
 	public void onSaveInstanceState(Bundle outState) {
 		super.onSaveInstanceState(outState);
-		outState.putSerializable(Constants.FRAGMENT_LIST, (Serializable) entries);
-		outState.putSerializable(Constants.FRAGMENT_LIST2, (Serializable) albums);
-		outState.putSerializable(Constants.FRAGMENT_EXTRA, (Serializable) artistInfo);
+        viewModel.setEntries(entries);
+        viewModel.setAlbums(albums);
+        viewModel.setArtistInfo(artistInfo);
 	}
 
 	@Override
@@ -165,14 +172,10 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 				id = childId;
 				lookupParent = true;
 			}
-			if(entries == null) {
-				entries = (List<Entry>) args.getSerializable(Constants.FRAGMENT_LIST);
-				albums = (List<Entry>) args.getSerializable(Constants.FRAGMENT_LIST2);
 
-				if(albums == null) {
-					albums = new ArrayList<Entry>();
-				}
-			}
+            if(albums == null) {
+                albums = new ArrayList<Entry>();
+            }
 		}
 
 		rootView = inflater.inflate(R.layout.abstract_recycler_fragment, container, false);
@@ -224,7 +227,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 	public void onCreateOptionsMenu(Menu menu, MenuInflater menuInflater) {
 		if(licenseValid == null) {
 			menuInflater.inflate(R.menu.empty, menu);
-		} else if(albumListType != null && !"starred".equals(albumListType)) {
+		} else if(albumListType != null && !albumListType.startsWith("starred") && !albumListType.startsWith("random")) {
 			menuInflater.inflate(R.menu.select_album_list, menu);
 		} else if(artist && !showAll) {
 			menuInflater.inflate(R.menu.select_album, menu);
@@ -258,6 +261,12 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 				if(!prefs.getBoolean(Constants.PREFERENCES_KEY_MENU_PLAY_LAST, true)) {
 					menu.setGroupVisible(R.id.hide_play_last, false);
 				}
+
+                if (albumListType.startsWith("random")) {
+                    menu.removeItem(R.id.menu_download);
+                    menu.removeItem(R.id.menu_cache);
+                    menu.removeItem(R.id.menu_delete);
+                }
 			} else {
 				if(Util.isOffline(context)) {
 					menuInflater.inflate(R.menu.select_podcast_episode_offline, menu);
@@ -272,7 +281,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 			}
 		}
 
-		if("starred".equals(albumListType)) {
+		if(albumListType.startsWith("starred")) {
 			menuInflater.inflate(R.menu.unstar, menu);
 		}
 	}
@@ -359,7 +368,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 
 			onSongPress(Arrays.asList(entry), entry, false);
 		} else {
-			onSongPress(entries, entry, albumListType == null || "starred".equals(albumListType));
+			onSongPress(entries, entry, albumListType == null || "starredsongs".equals(albumListType) || "randomsongs".equals(albumListType));
 		}
 	}
 
@@ -546,13 +555,15 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 			setTitle(R.string.main_albums_recent);
 		} else if ("frequent".equals(albumListType)) {
 			setTitle(R.string.main_albums_frequent);
-		} else if ("starred".equals(albumListType)) {
+		} else if ("starredalbums".equals(albumListType)) {
 			setTitle(R.string.main_albums_starred);
-		} else if("genres".equals(albumListType) || "years".equals(albumListType)) {
+		} else if ("starredsongs".equals(albumListType)) {
+            setTitle(R.string.main_songs_starred);
+        } else if("genres".equals(albumListType) || "years".equals(albumListType)) {
 			setTitle(albumListExtra);
 		} else if("alphabeticalByName".equals(albumListType)) {
 			setTitle(R.string.main_albums_alphabetical);
-		} if (MainFragment.SONGS_NEWEST.equals(albumListType)) {
+		} else if (MainFragment.SONGS_NEWEST.equals(albumListType)) {
 			setTitle(R.string.main_songs_newest);
 		} else if (MainFragment.SONGS_TOP_PLAYED.equals(albumListType)) {
 			setTitle(R.string.main_songs_top_played);
@@ -566,9 +577,12 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 			@Override
 			protected MusicDirectory load(MusicService service) throws Exception {
 				MusicDirectory result;
-				if ("starred".equals(albumListType)) {
-					result = service.getStarredList(context, this);
-				} else if(("genres".equals(albumListType) && ServerInfo.checkServerVersion(context, "1.10.0")) || "years".equals(albumListType)) {
+                StarredRepository starredRepository = new StarredRepository(service, context);
+				if ("starredalbums".equals(albumListType)) {
+					result = starredRepository.getStarredAlbums(this);
+				} else if ("starredsongs".equals(albumListType)) {
+                    result = starredRepository.getStarredSongs(this);
+                } else if(("genres".equals(albumListType) && ServerInfo.checkServerVersion(context, "1.10.0")) || "years".equals(albumListType)) {
 					result = service.getAlbumList(albumListType, albumListExtra, size, 0, refresh, context, this);
 					if(result.getChildrenSize() == 0 && "genres".equals(albumListType)) {
 						SelectDirectoryFragment.this.albumListType = "genres-songs";
@@ -577,7 +591,10 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 				} else if("genres".equals(albumListType) || "genres-songs".equals(albumListType)) {
 					result = service.getSongsByGenre(albumListExtra, size, 0, context, this);
 				} else if("randomsongs".equals(albumListType)) {
-					result = service.getRandomTracks(size,context, this);
+                    // Get users desired random playlist size
+                    SharedPreferences prefs = Util.getPreferences(context);
+                    int listSize = Math.max(1, Integer.parseInt(prefs.getString(Constants.PREFERENCES_KEY_RANDOM_SIZE, "20")));
+					result = service.getRandomSongs(listSize,context, this);
 				}  else if(albumListType.indexOf(MainFragment.SONGS_LIST_PREFIX) != -1) {
 					result = service.getSongList(albumListType, size, 0, context, this);
 				} else {
@@ -691,7 +708,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 			recyclerView.setVisibility(View.VISIBLE);
 		}
 
-		if(albumListType == null || "starred".equals(albumListType)) {
+		if(albumListType == null || albumListType.startsWith("starred")) {
 			entryGridAdapter = new EntryGridAdapter(context, entries, getImageLoader(), largeAlbums);
 			entryGridAdapter.setRemoveFromPlaylist(playlistId != null);
 		} else {
@@ -823,7 +840,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 	private void playAll(final boolean shuffle, final boolean append, final boolean playNext) {
 		boolean hasSubFolders = albums != null && !albums.isEmpty();
 
-		if (hasSubFolders && (id != null || share != null || "starred".equals(albumListType))) {
+		if (hasSubFolders && (id != null || share != null || "starredalbums".equals(albumListType))) {
 			downloadRecursively(id, false, append, !append, shuffle, false, playNext);
 		} else if(hasSubFolders && albumListType != null) {
 			downloadRecursively(albums, shuffle, append, playNext);
@@ -967,7 +984,7 @@ public class SelectDirectoryFragment extends SubsonicFragment implements Section
 	@Override
 	protected void toggleSelectedStarred() {
 		UpdateHelper.OnStarChange onStarChange = null;
-		if(albumListType != null && "starred".equals(albumListType)) {
+		if(albumListType != null && albumListType.startsWith("starred")) {
 			onStarChange = new UpdateHelper.OnStarChange() {
 				@Override
 				public void starChange(boolean starred) {
