@@ -23,6 +23,8 @@ import android.content.DialogInterface;
 import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.pm.PackageManager;
+import android.graphics.Color;
+import android.graphics.drawable.GradientDrawable;
 import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
@@ -63,6 +65,7 @@ import github.paroj.dsub2000.util.SyncUtil;
 import github.paroj.dsub2000.util.Util;
 import github.paroj.dsub2000.util.compat.GoogleCompat;
 import github.paroj.dsub2000.view.CacheLocationPreference;
+import github.paroj.dsub2000.view.ColorPickerPreference;
 import github.paroj.dsub2000.view.ErrorDialog;
 import github.paroj.dsub2000.view.EditPasswordPreference;
 
@@ -72,6 +75,9 @@ public class SettingsFragment extends PreferenceCompatFragment implements Shared
 	private final Map<String, ServerSettings> serverSettings = new LinkedHashMap<String, ServerSettings>();
 	private boolean testingConnection;
 	private ListPreference theme;
+    private CheckBoxPreference colorActionBar;
+    private ColorPickerPreference actionBarColor;
+    private ColorPickerPreference actionBarColorNowPlaying;
 	private ListPreference maxBitrateWifi;
 	private ListPreference maxBitrateMobile;
 	private ListPreference maxVideoBitrateWifi;
@@ -210,6 +216,12 @@ public class SettingsFragment extends PreferenceCompatFragment implements Shared
 					ActivityCompat.requestPermissions(context, new String[]{ Manifest.permission.ACCESS_COARSE_LOCATION }, SubsonicActivity.PERMISSIONS_REQUEST_LOCATION);
 				}
 			}
+            if (actionBarColor != null && actionBarColorNowPlaying != null) {
+                setColorIcon(actionBarColor, -1);
+                setColorIcon(actionBarColorNowPlaying, -1);
+                sharedPreferences.edit().putInt(Constants.PREFERENCES_KEY_ACTION_BAR_COLOR, -1).commit();
+                sharedPreferences.edit().putInt(Constants.PREFERENCES_KEY_ACTION_BAR_NOW_PLAYING_COLOR, -1).commit();
+            }
 		} else if(Constants.PREFERENCES_KEY_DLNA_CASTING_ENABLED.equals(key)) {
 			DownloadService downloadService = DownloadService.getInstance();
 			if(downloadService != null) {
@@ -222,7 +234,23 @@ public class SettingsFragment extends PreferenceCompatFragment implements Shared
 					mediaRouter.removeDLNAProvider();
 				}
 			}
-		}
+        } else if (Constants.PREFERENCES_KEY_COLOR_ACTION_BAR.equals(key)) {
+            if (actionBarColor != null && actionBarColorNowPlaying != null) {
+                boolean isActionBarColored = sharedPreferences.getBoolean(key, true);
+                actionBarColor.setEnabled(isActionBarColored);
+                actionBarColorNowPlaying.setEnabled(isActionBarColored);
+            }
+        } else if (Constants.PREFERENCES_KEY_ACTION_BAR_COLOR.equals(key)) {
+            if (actionBarColor != null) {
+                int color = sharedPreferences.getInt(key, -1);
+                setColorIcon(actionBarColor, color);
+            }
+        } else if (Constants.PREFERENCES_KEY_ACTION_BAR_NOW_PLAYING_COLOR.equals(key)) {
+            if (actionBarColorNowPlaying != null) {
+                int color = sharedPreferences.getInt(key, -1);
+                setColorIcon(actionBarColorNowPlaying, color);
+            }
+        }
 
 		scheduleBackup();
 	}
@@ -265,6 +293,9 @@ public class SettingsFragment extends PreferenceCompatFragment implements Shared
 		replayGainUntagged = this.findPreference(Constants.PREFERENCES_KEY_REPLAY_GAIN_UNTAGGED);
 		cacheSize = (EditTextPreference) this.findPreference(Constants.PREFERENCES_KEY_CACHE_SIZE);
 		openToTab = (ListPreference) this.findPreference(Constants.PREFERENCES_KEY_OPEN_TO_TAB);
+        colorActionBar = (CheckBoxPreference) this.findPreference(Constants.PREFERENCES_KEY_COLOR_ACTION_BAR);
+        actionBarColor = (ColorPickerPreference) this.findPreference(Constants.PREFERENCES_KEY_ACTION_BAR_COLOR);
+        actionBarColorNowPlaying = (ColorPickerPreference) this.findPreference(Constants.PREFERENCES_KEY_ACTION_BAR_NOW_PLAYING_COLOR);
 
 		settings = Util.getPreferences(context);
 		serverCount = settings.getInt(Constants.PREFERENCES_KEY_SERVER_COUNT, 1);
@@ -370,6 +401,17 @@ public class SettingsFragment extends PreferenceCompatFragment implements Shared
 		SharedPreferences prefs = Util.getPreferences(context);
 		prefs.registerOnSharedPreferenceChangeListener(this);
 
+        if (actionBarColor != null && actionBarColorNowPlaying != null) {
+            boolean isActionBarColored = prefs.getBoolean(Constants.PREFERENCES_KEY_COLOR_ACTION_BAR, true);
+            actionBarColor.setEnabled(isActionBarColored);
+            actionBarColorNowPlaying.setEnabled(isActionBarColored);
+
+            int actionBarColorInt = prefs.getInt(Constants.PREFERENCES_KEY_ACTION_BAR_COLOR, -1);
+            int actionBarColorNowPlayingInt = prefs.getInt(Constants.PREFERENCES_KEY_ACTION_BAR_NOW_PLAYING_COLOR, -1);
+            setColorIcon(actionBarColor, actionBarColorInt);
+            setColorIcon(actionBarColorNowPlaying, actionBarColorNowPlayingInt);
+        }
+
 		update();
 	}
 
@@ -476,6 +518,24 @@ public class SettingsFragment extends PreferenceCompatFragment implements Shared
 		}
 	}
 
+    private void setColorIcon(Preference preference, int color) {
+        if (color == -1) {
+            preference.setIcon(null);
+        } else {
+            int sizeDp = 24;
+            float scale = getResources().getDisplayMetrics().density;
+            int sizePx = (int) (sizeDp * scale + 0.5f);
+
+            GradientDrawable drawable = new GradientDrawable();
+            drawable.setShape(GradientDrawable.RECTANGLE);
+            drawable.setColor(color);
+            drawable.setCornerRadius(6f);
+            drawable.setSize(sizePx, sizePx);
+            drawable.setStroke(2, Color.DKGRAY);
+            preference.setIcon(drawable);
+        }
+    }
+
 	private PreferenceScreen addServer(final int instance) {
 		final PreferenceScreen screen = this.getPreferenceManager().createPreferenceScreen(context);
 		screen.setKey(Constants.PREFERENCES_KEY_SERVER_KEY + instance);
